@@ -28,6 +28,17 @@
 #include <gdk/gdk.h>
 #include "gdkalias.h"
 
+/**
+ * gdk_rectangle_union:
+ * @src1: a #GdkRectangle
+ * @src2: a #GdkRectangle
+ * @dest: return location for the union of @src1 and @src2
+ *
+ * Calculates the union of two rectangles.
+ * The union of rectangles @src1 and @src2 is the smallest rectangle which
+ * includes both @src1 and @src2 within it.
+ * It is allowed for @dest to be the same as either @src1 or @src2.
+ */
 void
 gdk_rectangle_union (GdkRectangle *src1,
 		     GdkRectangle *src2,
@@ -47,6 +58,21 @@ gdk_rectangle_union (GdkRectangle *src1,
   dest->y = dest_y;
 }
 
+/**
+ * gdk_rectangle_intersect:
+ * @src1: a #GdkRectangle
+ * @src2: a #GdkRectangle
+ * @dest: return location for the intersection of @src1 and @src2, or %NULL
+ *
+ * Calculates the intersection of two rectangles. It is allowed for
+ * @dest to be the same as either @src1 or @src2. If the rectangles 
+ * do not intersect, @dest's width and height is set to 0 and its x 
+ * and y values are undefined. If you are only interested in whether
+ * the rectangles intersect, but not in the intersecting area itself,
+ * pass %NULL for @dest.
+ *
+ * Returns: %TRUE if the rectangles intersect.
+ */
 gboolean
 gdk_rectangle_intersect (GdkRectangle *src1,
 			 GdkRectangle *src2,
@@ -58,7 +84,6 @@ gdk_rectangle_intersect (GdkRectangle *src1,
 
   g_return_val_if_fail (src1 != NULL, FALSE);
   g_return_val_if_fail (src2 != NULL, FALSE);
-  g_return_val_if_fail (dest != NULL, FALSE);
 
   return_val = FALSE;
 
@@ -69,13 +94,16 @@ gdk_rectangle_intersect (GdkRectangle *src1,
 
   if (dest_w > 0 && dest_h > 0)
     {
-      dest->x = dest_x;
-      dest->y = dest_y;
-      dest->width = dest_w;
-      dest->height = dest_h;
+      if (dest)
+        {
+          dest->x = dest_x;
+          dest->y = dest_y;
+          dest->width = dest_w;
+          dest->height = dest_h;
+        }
       return_val = TRUE;
     }
-  else
+  else if (dest)
     {
       dest->width = 0;
       dest->height = 0;
@@ -99,7 +127,7 @@ gdk_rectangle_get_type (void)
   static GType our_type = 0;
   
   if (our_type == 0)
-    our_type = g_boxed_type_register_static ("GdkRectangle",
+    our_type = g_boxed_type_register_static (g_intern_static_string ("GdkRectangle"),
 					     (GBoxedCopyFunc)gdk_rectangle_copy,
 					     (GBoxedFreeFunc)g_free);
   return our_type;
